@@ -1,11 +1,10 @@
-
 import timeit
 
 
-def autotimeit(stmt, setup='pass', repeat=3, mintime=0.2):
+def autotimeit(stmt, setup="pass", repeat=3, mintime=0.2):
     timer = timeit.Timer(stmt, setup)
     number, time1 = autoscaler(timer, mintime)
-    time2 = timer.repeat(repeat=repeat-1, number=number)
+    time2 = timer.repeat(repeat=repeat - 1, number=number)
     return min(time2 + [time1]) / number
 
 
@@ -16,4 +15,4 @@ def autoscaler(timer, mintime):
         if time > mintime:
             return number, time
         number *= 10
-    raise RuntimeError('function is too fast to test')
+    raise RuntimeError("function is too fast to test")
