@@ -1,3 +1,5 @@
+// Copyright 2010-2019 Keith Goodman
+// Copyright 2019 Bottleneck Developers
 #include "bottleneck.h"
 #include "iterators.h"
 
@@ -46,8 +48,7 @@ nonreducer_axis(char *name,
 #define B(dtype, i) AX(dtype, i) /* used by PARTITION */
 
 /* dtype = [['float64'], ['float32'], ['int64'], ['int32']] */
-NRA(partition, DTYPE0)
-{
+NRA(partition, DTYPE0) {
     npy_intp i;
     npy_intp j, l, r, k;
     iter it;
@@ -121,8 +122,7 @@ NRA_MAIN(partition, PARSE_PARTITION)
                     ytmp = YX(dtype1, k); \
                     YX(dtype1, k) = YX(dtype1, l); \
                     YX(dtype1, l) = ytmp; \
-                } \
-                else { \
+                } else { \
                     B[k] = ar; \
                     B[r] = ak; \
                     ytmp = YX(dtype1, k); \
@@ -130,8 +130,7 @@ NRA_MAIN(partition, PARSE_PARTITION)
                     YX(dtype1, r) = ytmp; \
                 } \
             } \
-        } \
-        else { \
+        } else { \
             if (ak > ar) { \
                 if (al > ar) { \
                     B[k] = al; \
@@ -139,8 +138,7 @@ NRA_MAIN(partition, PARSE_PARTITION)
                     ytmp = YX(dtype1, k); \
                     YX(dtype1, k) = YX(dtype1, l); \
                     YX(dtype1, l) = ytmp; \
-                } \
-                else { \
+                } else { \
                     B[k] = ar; \
                     B[r] = ak; \
                     ytmp = YX(dtype1, k); \
@@ -163,8 +161,7 @@ NRA_MAIN(partition, PARSE_PARTITION)
 
 /* dtype = [['float64', 'intp'], ['float32', 'intp'],
             ['int64',   'intp'], ['int32',   'intp']] */
-NRA(argpartition, DTYPE0)
-{
+NRA(argpartition, DTYPE0) {
     npy_intp i;
     PyObject *y = PyArray_EMPTY(PyArray_NDIM(a), PyArray_SHAPE(a),
                                 NPY_DTYPE1, 0);
@@ -200,8 +197,7 @@ NRA_MAIN(argpartition, PARSE_PARTITION)
 
 /* dtype = [['float64', 'float64', 'intp'], ['float32', 'float64', 'intp'],
             ['int64',   'float64', 'intp'], ['int32',   'float64', 'intp']] */
-NRA(rankdata, DTYPE0)
-{
+NRA(rankdata, DTYPE0) {
     Py_ssize_t j=0, k, idx, dupcount=0, i;
     npy_DTYPE1 old, new, averank, sumranks = 0;
 
@@ -217,8 +213,7 @@ NRA(rankdata, DTYPE0)
         Py_ssize_t size = PyArray_SIZE((PyArrayObject *)y);
         npy_DTYPE1 *py = (npy_DTYPE1 *)PyArray_DATA(a);
         for (i = 0; i < size; i++) YPP = BN_NAN;
-    }
-    else {
+    } else {
         WHILE {
             idx = ZX(DTYPE2, 0);
             old = AX(DTYPE0, idx);
@@ -264,8 +259,7 @@ NRA_MAIN(rankdata, PARSE_RANKDATA)
 /* nanrankdata ----------------------------------------------------------- */
 
 /* dtype = [['float64', 'float64', 'intp'], ['float32', 'float64', 'intp']] */
-NRA(nanrankdata, DTYPE0)
-{
+NRA(nanrankdata, DTYPE0) {
     Py_ssize_t j=0, k, idx, dupcount=0, i;
     npy_DTYPE1 old, new, averank, sumranks = 0;
 
@@ -281,8 +275,7 @@ NRA(nanrankdata, DTYPE0)
         Py_ssize_t size = PyArray_SIZE((PyArrayObject *)y);
         npy_DTYPE1 *py = (npy_DTYPE1 *)PyArray_DATA(a);
         for (i = 0; i < size; i++) YPP = BN_NAN;
-    }
-    else {
+    } else {
         WHILE {
             idx = ZX(DTYPE2, 0);
             old = AX(DTYPE0, idx);
@@ -301,8 +294,7 @@ NRA(nanrankdata, DTYPE0)
                             idx = ZX(DTYPE2, j);
                             YX(DTYPE1, idx) = averank;
                         }
-                    }
-                    else {
+                    } else {
                         idx = ZX(DTYPE2, i);
                         YX(DTYPE1, idx) = BN_NAN;
                     }
@@ -319,8 +311,7 @@ NRA(nanrankdata, DTYPE0)
                     idx = ZX(DTYPE2, j);
                     YX(DTYPE1, idx) = averank;
                 }
-            }
-            else {
+            } else {
                 idx = ZX(DTYPE2, LENGTH - 1);
                 YX(DTYPE1, idx) = BN_NAN;
             }
@@ -335,8 +326,7 @@ NRA(nanrankdata, DTYPE0)
 /* dtype end */
 
 static PyObject *
-nanrankdata(PyObject *self, PyObject *args, PyObject *kwds)
-{
+nanrankdata(PyObject *self, PyObject *args, PyObject *kwds) {
     return nonreducer_axis("nanrankdata",
                            args,
                            kwds,
@@ -351,8 +341,7 @@ nanrankdata(PyObject *self, PyObject *args, PyObject *kwds)
 /* push ------------------------------------------------------------------ */
 
 /* dtype = [['float64'], ['float32']] */
-NRA(push, DTYPE0)
-{
+NRA(push, DTYPE0) {
     npy_intp index;
     npy_DTYPE0 ai, ai_last, n_float;
     PyObject *y = PyArray_Copy(a);
@@ -371,8 +360,7 @@ NRA(push, DTYPE0)
             if (ai == ai) {
                 ai_last = ai;
                 index = INDEX;
-            }
-            else {
+            } else {
                 if (INDEX - index <= n_float) {
                     AI(DTYPE0) = ai_last;
                 }
@@ -386,8 +374,7 @@ NRA(push, DTYPE0)
 /* dtype end */
 
 /* dtype = [['int64'], ['int32']] */
-NRA(push, DTYPE0)
-{
+NRA(push, DTYPE0) {
     PyObject *y = PyArray_Copy(a);
     return y;
 }
@@ -405,22 +392,21 @@ PyObject *pystr_axis = NULL;
 
 static int
 intern_strings(void) {
-    pystr_a = PyString_InternFromString("a");
-    pystr_n = PyString_InternFromString("n");
-    pystr_kth = PyString_InternFromString("kth");
-    pystr_axis = PyString_InternFromString("axis");
+    pystr_a = PyUnicode_InternFromString("a");
+    pystr_n = PyUnicode_InternFromString("n");
+    pystr_kth = PyUnicode_InternFromString("kth");
+    pystr_axis = PyUnicode_InternFromString("axis");
     return pystr_a && pystr_n && pystr_axis;
 }
 
 /* nonreducer_axis ------------------------------------------------------- */
 
-static BN_INLINE int
+static inline int
 parse_partition(PyObject *args,
                 PyObject *kwds,
                 PyObject **a,
                 PyObject **n,
-                PyObject **axis)
-{
+                PyObject **axis) {
     const Py_ssize_t nargs = PyTuple_GET_SIZE(args);
     const Py_ssize_t nkwds = kwds == NULL ? 0 : PyDict_Size(kwds);
     if (nkwds) {
@@ -468,8 +454,7 @@ parse_partition(PyObject *args,
             TYPE_ERR("too many arguments");
             return 0;
         }
-    }
-    else {
+    } else {
         switch (nargs) {
             case 3:
                 *axis = PyTuple_GET_ITEM(args, 2);
@@ -487,12 +472,11 @@ parse_partition(PyObject *args,
 
 }
 
-static BN_INLINE int
+static inline int
 parse_rankdata(PyObject *args,
                PyObject *kwds,
                PyObject **a,
-               PyObject **axis)
-{
+               PyObject **axis) {
     const Py_ssize_t nargs = PyTuple_GET_SIZE(args);
     const Py_ssize_t nkwds = kwds == NULL ? 0 : PyDict_Size(kwds);
     if (nkwds) {
@@ -532,8 +516,7 @@ parse_rankdata(PyObject *args,
             TYPE_ERR("too many arguments");
             return 0;
         }
-    }
-    else {
+    } else {
         switch (nargs) {
             case 2:
                 *axis = PyTuple_GET_ITEM(args, 1);
@@ -550,13 +533,12 @@ parse_rankdata(PyObject *args,
 
 }
 
-static BN_INLINE int
+static inline int
 parse_push(PyObject *args,
            PyObject *kwds,
            PyObject **a,
            PyObject **n,
-           PyObject **axis)
-{
+           PyObject **axis) {
     const Py_ssize_t nargs = PyTuple_GET_SIZE(args);
     const Py_ssize_t nkwds = kwds == NULL ? 0 : PyDict_Size(kwds);
     if (nkwds) {
@@ -603,8 +585,7 @@ parse_push(PyObject *args,
             TYPE_ERR("too many arguments");
             return 0;
         }
-    }
-    else {
+    } else {
         switch (nargs) {
             case 3:
                 *axis = PyTuple_GET_ITEM(args, 2);
@@ -631,14 +612,14 @@ nonreducer_axis(char *name,
                 nra_t nra_float32,
                 nra_t nra_int64,
                 nra_t nra_int32,
-                parse_type parse)
-{
+                parse_type parse) {
 
     int n;
     int axis;
     int dtype;
 
     PyArrayObject *a;
+    PyObject *y;
 
     PyObject *a_obj = NULL;
     PyObject *n_obj = NULL;
@@ -648,26 +629,23 @@ nonreducer_axis(char *name,
         if (!parse_partition(args, kwds, &a_obj, &n_obj, &axis_obj)) {
             return NULL;
         }
-    }
-    else if (parse == PARSE_RANKDATA) {
+    } else if (parse == PARSE_RANKDATA) {
         if (!parse_rankdata(args, kwds, &a_obj, &axis_obj)) {
             return NULL;
         }
-    }
-    else if (parse == PARSE_PUSH) {
+    } else if (parse == PARSE_PUSH) {
         if (!parse_push(args, kwds, &a_obj, &n_obj, &axis_obj)) {
             return NULL;
         }
-    }
-    else {
+    } else {
         RUNTIME_ERR("Unknown parse type; please report error.");
     }
 
     /* convert to array if necessary */
-    if PyArray_Check(a_obj) {
+    if (PyArray_Check(a_obj)) {
         a = (PyArrayObject *)a_obj;
-    }
-    else {
+        Py_INCREF(a);
+    } else {
         a = (PyArrayObject *)PyArray_FROM_O(a_obj);
         if (a == NULL) {
             return NULL;
@@ -675,7 +653,8 @@ nonreducer_axis(char *name,
     }
 
     /* check for byte swapped input array */
-    if PyArray_ISBYTESWAPPED(a) {
+    if (PyArray_ISBYTESWAPPED(a)) {
+        Py_DECREF(a);
         return slow(name, args, kwds);
     }
 
@@ -686,68 +665,73 @@ nonreducer_axis(char *name,
             if (axis < 0) {
                 PyErr_Format(PyExc_ValueError,
                              "axis(=%d) out of bounds", axis);
-                return NULL;
+                goto error;
             }
-        }
-        else {
+        } else {
             if (PyArray_NDIM(a) != 1) {
                 a = (PyArrayObject *)PyArray_Ravel(a, NPY_CORDER);
             }
             axis = 0;
         }
-    }
-    else if (axis_obj == Py_None) {
+    } else if (axis_obj == Py_None) {
         if (parse == PARSE_PUSH) {
             VALUE_ERR("`axis` cannot be None");
-            return NULL;
+            goto error;
         }
         if (PyArray_NDIM(a) != 1) {
             a = (PyArrayObject *)PyArray_Ravel(a, NPY_CORDER);
         }
         axis = 0;
-    }
-    else {
+    } else {
         axis = PyArray_PyIntAsInt(axis_obj);
         if (error_converting(axis)) {
             TYPE_ERR("`axis` must be an integer");
-            return NULL;
+            goto error;
         }
         if (axis < 0) {
             axis += PyArray_NDIM(a);
             if (axis < 0) {
                 PyErr_Format(PyExc_ValueError,
                              "axis(=%d) out of bounds", axis);
-                return NULL;
+                goto error;
             }
-        }
-        else if (axis >= PyArray_NDIM(a)) {
+        } else if (axis >= PyArray_NDIM(a)) {
             PyErr_Format(PyExc_ValueError, "axis(=%d) out of bounds", axis);
-            return NULL;
+            goto error;
         }
     }
 
-    /* ddof */
+    /* n */
     if (n_obj == NULL) {
         n = -1;
-    }
-    else {
+    } else if (parse == PARSE_PUSH && n_obj == Py_None) {
+        n = -1;
+    } else {
         n = PyArray_PyIntAsInt(n_obj);
         if (error_converting(n)) {
             TYPE_ERR("`n` must be an integer");
-            return NULL;
+            goto error;
         }
         if (n < 0 && parse == PARSE_PUSH) {
             VALUE_ERR("`n` must be nonnegative");
-            return NULL;
+            goto error;
         }
     }
 
     dtype = PyArray_TYPE(a);
-    if      (dtype == NPY_float64) return nra_float64(a, axis, n);
-    else if (dtype == NPY_float32) return nra_float32(a, axis, n);
-    else if (dtype == NPY_int64)   return nra_int64(a, axis, n);
-    else if (dtype == NPY_int32)   return nra_int32(a, axis, n);
-    else                           return slow(name, args, kwds);
+    if      (dtype == NPY_float64) y = nra_float64(a, axis, n);
+    else if (dtype == NPY_float32) y = nra_float32(a, axis, n);
+    else if (dtype == NPY_int64)   y = nra_int64(a, axis, n);
+    else if (dtype == NPY_int32)   y = nra_int32(a, axis, n);
+    else                           y = slow(name, args, kwds);
+
+    Py_DECREF(a);
+
+    return y;
+
+error:
+    Py_DECREF(a);
+    return NULL;
 
 }
 
@@ -780,7 +764,7 @@ a : array_like
     Input array. If `a` is not an array, a conversion is attempted.
 kth : int
     The value of the element at index `kth` will be in its sorted
-    postion. Smaller (larger) or equal values will be to the left
+    position. Smaller (larger) or equal values will be to the left
     (right) of index `kth`.
 axis : {int, None}, optional
     Axis along which the partition is performed. The default
@@ -845,7 +829,7 @@ a : array_like
     Input array. If `a` is not an array, a conversion is attempted.
 kth : int
     The value of the element at index `kth` will be in its sorted
-    postion. Smaller (larger) or equal values will be to the left
+    position. Smaller (larger) or equal values will be to the left
     (right) of index `kth`.
 axis : {int, None}, optional
     Axis along which the partition is performed. The default (axis=-1)
@@ -1028,7 +1012,6 @@ nra_methods[] = {
 };
 
 
-#if PY_MAJOR_VERSION >= 3
 static struct PyModuleDef
 nra_def = {
    PyModuleDef_HEAD_INIT,
@@ -1037,27 +1020,21 @@ nra_def = {
    -1,
    nra_methods
 };
-#endif
 
 
 PyMODINIT_FUNC
-#if PY_MAJOR_VERSION >= 3
-#define RETVAL m
 PyInit_nonreduce_axis(void)
-#else
-#define RETVAL
-initnonreduce_axis(void)
-#endif
 {
-    #if PY_MAJOR_VERSION >=3
-        PyObject *m = PyModule_Create(&nra_def);
-    #else
-        PyObject *m = Py_InitModule3("nonreduce_axis", nra_methods, nra_doc);
+    PyObject *m = PyModule_Create(&nra_def);
+    if (m == NULL) return NULL;
+
+    #ifdef Py_GIL_DISABLED
+        PyUnstable_Module_SetGIL(m, Py_MOD_GIL_NOT_USED);
     #endif
-    if (m == NULL) return RETVAL;
+
     import_array();
     if (!intern_strings()) {
-        return RETVAL;
+        return NULL;
     }
-    return RETVAL;
+    return m;
 }
